@@ -31,6 +31,7 @@ from nautobot.extras.models import (
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
+    JobLogEntry,
     JobResult,
     ObjectChange,
     Relationship,
@@ -49,7 +50,7 @@ from nautobot.extras.jobs import get_job, get_jobs, run_job
 from nautobot.extras.utils import get_worker_count
 from nautobot.utilities.exceptions import CeleryWorkerNotRunningException
 from nautobot.utilities.utils import copy_safe_request, count_related
-from . import serializers
+from . import nested_serializers, serializers
 
 
 class ExtrasRootView(APIRootView):
@@ -430,6 +431,16 @@ class JobViewSet(viewsets.ViewSet):
 #
 
 
+class JobLogEntryViewSet(ReadOnlyModelViewSet):
+    """
+    Retrieve a list of job log entries.
+    """
+
+    queryset = JobLogEntry.objects.prefetch_related("job_result")
+    serializer_class = serializers.JobLogEntrySerializer
+    filterset_class = filters.JobLogEntryFilterSet
+
+
 class JobResultViewSet(ModelViewSet):
     """
     Retrieve a list of job results
@@ -439,6 +450,13 @@ class JobResultViewSet(ModelViewSet):
     serializer_class = serializers.JobResultSerializer
     filterset_class = filters.JobResultFilterSet
 
+    @action(detail=True)
+    def logs(self, request, pk=None):
+        job_result = self.get_object()
+        logs = job_result.logs.all()
+        serializer = nested_serializers.NestedJobLogEntrySerializer(logs, context={"request": request}, many=True)
+        return Response(serializer.data)
+
 
 #
 # Scheduled Jobs
