@@ -6,6 +6,7 @@ from datetime import datetime, date
 from django import forms
 from django.db import transaction
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ObjectDoesNotExist
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.validators import RegexValidator, ValidationError
 from django.db import models
@@ -13,7 +14,7 @@ from django.urls import reverse
 from django.utils.safestring import mark_safe
 
 from nautobot.extras.choices import CustomFieldFilterLogicChoices, CustomFieldTypeChoices
-from nautobot.extras.models import ChangeLoggedModel
+from nautobot.extras.models import ChangeLoggedModel, ObjectChange
 from nautobot.extras.tasks import delete_custom_field_data, update_custom_field_choice_data
 from nautobot.extras.utils import FeatureQuery, extras_features
 from nautobot.core.fields import AutoSlugField
@@ -29,10 +30,9 @@ from nautobot.utilities.forms import (
 )
 from nautobot.utilities.querysets import RestrictedQuerySet
 from nautobot.utilities.templatetags.helpers import render_markdown
-from nautobot.utilities.utils import render_jinja2
+from nautobot.utilities.utils import render_jinja2, serialize_object
 from nautobot.utilities.validators import validate_regex
 
-
 logger = logging.getLogger(__name__)
 
 
@@ -197,7 +197,7 @@ class CustomFieldManager(models.Manager.from_queryset(RestrictedQuerySet)):
 
 
 @extras_features("webhooks")
-class CustomField(BaseModel):
+class CustomField(BaseModel, ChangeLoggedModel):
     content_types = models.ManyToManyField(
         to=ContentType,
         related_name="custom_fields",
@@ -489,7 +489,7 @@ class CustomField(BaseModel):
     "graphql",
     "webhooks",
 )
-class CustomFieldChoice(BaseModel):
+class CustomFieldChoice(BaseModel, ChangeLoggedModel):
     """
     The custom field choice is used to store the possible set of values for a selection type custom field
     """
@@ -565,3 +565,18 @@ class CustomFieldChoice(BaseModel):
                     raise models.ProtectedError(self, "Cannot delete this choice because it is in active use.")
 
         super().delete(*args, **kwargs)
+
+    def to_objectchange(self, action):
+        # Annotate the parent field
+        try:
+            field = self.field
+        except ObjectDoesNotExist:
+            # The parent field has already been deleted
+            field = None
+        return ObjectChange(
+            changed_object=self,
+            object_repr=str(self),
+            action=action,
+            related_object=field,
+            object_data=serialize_object(self),
+        )
