@@ -1,3 +1,4 @@
+from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.db.models import ProtectedError
@@ -8,7 +9,7 @@ from nautobot.dcim.filters import SiteFilterSet
 from nautobot.dcim.forms import SiteCSVForm
 from nautobot.dcim.models import Site, Rack, Device
 from nautobot.dcim.tables import SiteTable
-from nautobot.extras.choices import *
+from nautobot.extras.choices import CustomFieldTypeChoices, CustomFieldFilterLogicChoices
 from nautobot.extras.models import ComputedField, CustomField, CustomFieldChoice, Status
 from nautobot.utilities.tables import CustomFieldColumn
 from nautobot.utilities.testing import APITestCase, CeleryTestCase, TestCase
@@ -165,7 +166,10 @@ class CustomFieldManagerTest(TestCase):
         custom_field.content_types.set([content_type])
 
     def test_get_for_model(self):
-        self.assertEqual(CustomField.objects.get_for_model(Site).count(), 1)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(CustomField.objects.get_for_model(Site).count(), 2)
+        else:
+            self.assertEqual(CustomField.objects.get_for_model(Site).count(), 1)
         self.assertEqual(CustomField.objects.get_for_model(VirtualMachine).count(), 0)
 
 
@@ -237,6 +241,9 @@ class CustomFieldAPITest(APITestCase):
         cls.cf_multi_select.default = ["Foo", "Bar"]
         cls.cf_multi_select.save()
 
+        if "dummy_plugin" in settings.PLUGINS:
+            cls.cf_plugin_field = CustomField.objects.get(name="dummy-plugin-auto-custom-field")
+
         statuses = Status.objects.get_for_model(Site)
 
         # Create some sites
@@ -255,6 +262,8 @@ class CustomFieldAPITest(APITestCase):
             cls.cf_select.name: "Bar",
             cls.cf_multi_select.name: ["Bar", "Baz"],
         }
+        if "dummy_plugin" in settings.PLUGINS:
+            cls.sites[1]._custom_field_data[cls.cf_plugin_field.name] = "Custom value"
         cls.sites[1].save()
 
     def test_get_single_object_without_custom_field_data(self):
@@ -266,18 +275,20 @@ class CustomFieldAPITest(APITestCase):
 
         response = self.client.get(url, **self.header)
         self.assertEqual(response.data["name"], self.sites[0].name)
-        self.assertEqual(
-            response.data["custom_fields"],
-            {
-                "text_field": None,
-                "number_field": None,
-                "boolean_field": None,
-                "date_field": None,
-                "url_field": None,
-                "choice_field": None,
-                "multi_choice_field": None,
-            },
-        )
+        expected_data = {
+            "text_field": None,
+            "number_field": None,
+            "boolean_field": None,
+            "date_field": None,
+            "url_field": None,
+            "choice_field": None,
+            "multi_choice_field": None,
+        }
+        if "dummy_plugin" in settings.PLUGINS:
+            # A model directly instantiated via the ORM does NOT automatically receive custom field default values.
+            # This is arguably a bug.
+            expected_data["dummy-plugin-auto-custom-field"] = None
+        self.assertEqual(response.data["custom_fields"], expected_data)
 
     def test_get_single_object_with_custom_field_data(self):
         """
@@ -296,6 +307,11 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(response.data["custom_fields"]["url_field"], site2_cfvs["url_field"])
         self.assertEqual(response.data["custom_fields"]["choice_field"], site2_cfvs["choice_field"])
         self.assertEqual(response.data["custom_fields"]["multi_choice_field"], site2_cfvs["multi_choice_field"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(
+                response.data["custom_fields"]["dummy-plugin-auto-custom-field"],
+                site2_cfvs["dummy-plugin-auto-custom-field"],
+            )
 
     def test_create_single_object_with_defaults(self):
         """
@@ -321,6 +337,8 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(response_cf["url_field"], self.cf_url.default)
         self.assertEqual(response_cf["choice_field"], self.cf_select.default)
         self.assertEqual(response_cf["multi_choice_field"], self.cf_multi_select.default)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(response_cf["dummy-plugin-auto-custom-field"], self.cf_plugin_field.default)
 
         # Validate database data
         site = Site.objects.get(pk=response.data["id"])
@@ -331,6 +349,8 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(site.cf["url_field"], self.cf_url.default)
         self.assertEqual(site.cf["choice_field"], self.cf_select.default)
         self.assertEqual(site.cf["multi_choice_field"], self.cf_multi_select.default)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(site.cf["dummy-plugin-auto-custom-field"], self.cf_plugin_field.default)
 
     def test_create_single_object_with_values(self):
         """
@@ -350,6 +370,8 @@ class CustomFieldAPITest(APITestCase):
                 "multi_choice_field": ["Baz"],
             },
         }
+        if "dummy_plugin" in settings.PLUGINS:
+            data["custom_fields"]["dummy-plugin-auto-custom-field"] = "Custom value"
         url = reverse("dcim-api:site-list")
         self.add_permissions("dcim.add_site")
 
@@ -366,6 +388,8 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(response_cf["url_field"], data_cf["url_field"])
         self.assertEqual(response_cf["choice_field"], data_cf["choice_field"])
         self.assertEqual(response_cf["multi_choice_field"], data_cf["multi_choice_field"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(response_cf["dummy-plugin-auto-custom-field"], data_cf["dummy-plugin-auto-custom-field"])
 
         # Validate database data
         site = Site.objects.get(pk=response.data["id"])
@@ -376,6 +400,8 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(site.cf["url_field"], data_cf["url_field"])
         self.assertEqual(site.cf["choice_field"], data_cf["choice_field"])
         self.assertEqual(site.cf["multi_choice_field"], data_cf["multi_choice_field"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(site.cf["dummy-plugin-auto-custom-field"], data_cf["dummy-plugin-auto-custom-field"])
 
     def test_create_multiple_objects_with_defaults(self):
         """
@@ -417,6 +443,8 @@ class CustomFieldAPITest(APITestCase):
             self.assertEqual(response_cf["url_field"], self.cf_url.default)
             self.assertEqual(response_cf["choice_field"], self.cf_select.default)
             self.assertEqual(response_cf["multi_choice_field"], self.cf_multi_select.default)
+            if "dummy_plugin" in settings.PLUGINS:
+                self.assertEqual(response_cf["dummy-plugin-auto-custom-field"], self.cf_plugin_field.default)
 
             # Validate database data
             site = Site.objects.get(pk=response.data[i]["id"])
@@ -427,6 +455,8 @@ class CustomFieldAPITest(APITestCase):
             self.assertEqual(site.cf["url_field"], self.cf_url.default)
             self.assertEqual(site.cf["choice_field"], self.cf_select.default)
             self.assertEqual(site.cf["multi_choice_field"], self.cf_multi_select.default)
+            if "dummy_plugin" in settings.PLUGINS:
+                self.assertEqual(site.cf["dummy-plugin-auto-custom-field"], self.cf_plugin_field.default)
 
     def test_create_multiple_objects_with_values(self):
         """
@@ -441,6 +471,8 @@ class CustomFieldAPITest(APITestCase):
             "choice_field": "Bar",
             "multi_choice_field": ["Foo", "Bar"],
         }
+        if "dummy_plugin" in settings.PLUGINS:
+            custom_field_data["dummy-plugin-auto-custom-field"] = "Custom value"
         data = (
             {
                 "name": "Site 3",
@@ -479,6 +511,10 @@ class CustomFieldAPITest(APITestCase):
             self.assertEqual(response_cf["url_field"], custom_field_data["url_field"])
             self.assertEqual(response_cf["choice_field"], custom_field_data["choice_field"])
             self.assertEqual(response_cf["multi_choice_field"], custom_field_data["multi_choice_field"])
+            if "dummy_plugin" in settings.PLUGINS:
+                self.assertEqual(
+                    response_cf["dummy-plugin-auto-custom-field"], custom_field_data["dummy-plugin-auto-custom-field"]
+                )
 
             # Validate database data
             site = Site.objects.get(pk=response.data[i]["id"])
@@ -504,6 +540,10 @@ class CustomFieldAPITest(APITestCase):
                 site.cf["multi_choice_field"],
                 custom_field_data["multi_choice_field"],
             )
+            if "dummy_plugin" in settings.PLUGINS:
+                self.assertEqual(
+                    site.cf["dummy-plugin-auto-custom-field"], custom_field_data["dummy-plugin-auto-custom-field"]
+                )
 
     def test_update_single_object_with_values(self):
         """
@@ -533,6 +573,10 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(response_cf["url_field"], original_cfvs["url_field"])
         self.assertEqual(response_cf["choice_field"], original_cfvs["choice_field"])
         self.assertEqual(response_cf["multi_choice_field"], original_cfvs["multi_choice_field"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(
+                response_cf["dummy-plugin-auto-custom-field"], original_cfvs["dummy-plugin-auto-custom-field"]
+            )
 
         # Validate database data
         site.refresh_from_db()
@@ -546,6 +590,8 @@ class CustomFieldAPITest(APITestCase):
         self.assertEqual(site.cf["url_field"], original_cfvs["url_field"])
         self.assertEqual(site.cf["choice_field"], original_cfvs["choice_field"])
         self.assertEqual(site.cf["multi_choice_field"], original_cfvs["multi_choice_field"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(site.cf["dummy-plugin-auto-custom-field"], original_cfvs["dummy-plugin-auto-custom-field"])
 
     def test_minimum_maximum_values_validation(self):
         url = reverse("dcim-api:site-detail", kwargs={"pk": self.sites[1].pk})
@@ -616,6 +662,25 @@ class CustomFieldAPITest(APITestCase):
         response = self.client.patch(url, data, format="json", **self.header)
         self.assertHttpStatus(response, status.HTTP_200_OK)
 
+    def test_select_regex_validation(self):
+        url = reverse("extras-api:customfieldchoice-list")
+        self.add_permissions("extras.add_customfieldchoice")
+
+        self.cf_select.validation_regex = r"^[A-Z]{3}$"  # Three uppercase letters
+        self.cf_select.save()
+
+        data = {"field": self.cf_select.id, "value": "1234", "weight": 100}
+        response = self.client.post(url, data, format="json", **self.header)
+        self.assertHttpStatus(response, status.HTTP_400_BAD_REQUEST)
+
+        data = {"field": self.cf_select.id, "value": "abc", "weight": 100}
+        response = self.client.post(url, data, format="json", **self.header)
+        self.assertHttpStatus(response, status.HTTP_400_BAD_REQUEST)
+
+        data = {"field": self.cf_select.id, "value": "ABC", "weight": 100}
+        response = self.client.post(url, data, format="json", **self.header)
+        self.assertHttpStatus(response, status.HTTP_201_CREATED)
+
 
 class CustomFieldImportTest(TestCase):
     user_permissions = (
@@ -658,7 +723,7 @@ class CustomFieldImportTest(TestCase):
         Import a Site in CSV format, including a value for each CustomField.
         """
         data = (
-            (
+            [
                 "name",
                 "slug",
                 "status",
@@ -669,8 +734,8 @@ class CustomFieldImportTest(TestCase):
                 "cf_url",
                 "cf_select",
                 "cf_multiselect",
-            ),
-            (
+            ],
+            [
                 "Site 1",
                 "site-1",
                 "active",
@@ -681,8 +746,8 @@ class CustomFieldImportTest(TestCase):
                 "http://example.com/1",
                 "Choice A",
                 "Choice A",
-            ),
-            (
+            ],
+            [
                 "Site 2",
                 "site-2",
                 "active",
@@ -693,17 +758,25 @@ class CustomFieldImportTest(TestCase):
                 "http://example.com/2",
                 "Choice B",
                 '"Choice A,Choice B"',
-            ),
-            ("Site 3", "site-3", "active", "", "", "", "", "", "", ""),
+            ],
+            ["Site 3", "site-3", "active", "", "", "", "", "", "", ""],
         )
+        if "dummy_plugin" in settings.PLUGINS:
+            data[0].append("cf_dummy-plugin-auto-custom-field")
+            data[1].append("Custom value")
+            data[2].append("Another custom value")
+            data[3].append("")
         csv_data = "\n".join(",".join(row) for row in data)
 
-        response = self.client.post(reverse("dcim:site_import"), {"csv": csv_data})
+        response = self.client.post(reverse("dcim:site_import"), {"csv_data": csv_data})
         self.assertEqual(response.status_code, 200)
 
         # Validate data for site 1
         site1 = Site.objects.get(name="Site 1")
-        self.assertEqual(len(site1.cf), 7)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(len(site1.cf), 8)
+        else:
+            self.assertEqual(len(site1.cf), 7)
         self.assertEqual(site1.cf["text"], "ABC")
         self.assertEqual(site1.cf["integer"], 123)
         self.assertEqual(site1.cf["boolean"], True)
@@ -711,10 +784,15 @@ class CustomFieldImportTest(TestCase):
         self.assertEqual(site1.cf["url"], "http://example.com/1")
         self.assertEqual(site1.cf["select"], "Choice A")
         self.assertEqual(site1.cf["multiselect"], ["Choice A"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(site1.cf["dummy-plugin-auto-custom-field"], "Custom value")
 
         # Validate data for site 2
         site2 = Site.objects.get(name="Site 2")
-        self.assertEqual(len(site2.cf), 7)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(len(site2.cf), 8)
+        else:
+            self.assertEqual(len(site2.cf), 7)
         self.assertEqual(site2.cf["text"], "DEF")
         self.assertEqual(site2.cf["integer"], 456)
         self.assertEqual(site2.cf["boolean"], False)
@@ -722,6 +800,8 @@ class CustomFieldImportTest(TestCase):
         self.assertEqual(site2.cf["url"], "http://example.com/2")
         self.assertEqual(site2.cf["select"], "Choice B")
         self.assertEqual(site2.cf["multiselect"], ["Choice A", "Choice B"])
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(site2.cf["dummy-plugin-auto-custom-field"], "Another custom value")
 
         # No custom field data should be set for site 3
         site3 = Site.objects.get(name="Site 3")
@@ -1081,7 +1161,10 @@ class CustomFieldChoiceTest(TestCase):
 
     def test_custom_choice_deleted_with_field(self):
         self.cf.delete()
-        self.assertEqual(CustomField.objects.count(), 0)
+        if "dummy_plugin" in settings.PLUGINS:
+            self.assertEqual(CustomField.objects.count(), 1)  # custom field automatically added by the plugin
+        else:
+            self.assertEqual(CustomField.objects.count(), 0)
         self.assertEqual(CustomFieldChoice.objects.count(), 0)
 
 
